#include <stdio.h>
#include <stdlib.h>
#include <alloc.h>
#include <windows.h>

#include "bmp.h"


// open bmp file for read
BMPinfo *OpenReadBmpFile(char *filename)
{
	char str[80];
	DWORD n;
	BITMAPFILEHEADER header;
	BITMAPINFOHEADER info;
	BMPinfo *bmp = (BMPinfo *)malloc(sizeof(BMPinfo));

	bmp->file = fopen(filename, "rb");

	fread(&header, sizeof(header), 1, bmp->file);

	wsprintf(str, "type:%x, size:%ld, offset:%ld\n",
					 header.bfType, header.bfSize, header.bfOffBits);
	MessageBox(NULL, str, "Read BMP", MB_OK);

	fread(&info, sizeof(info), 1, bmp->file);

	wsprintf(str, "w:%ld, h:%ld, planes:%d, bits:%d, comp:%ld, size:%ld\n",
				 info.biWidth, info.biHeight, info.biPlanes, info.biBitCount,
				 info.biCompression, info.biSizeImage);
	MessageBox(NULL, str, "Read BMP", MB_OK);

	wsprintf(str, "xres:%ld, yres:%ld, used:%ld, imp:%ld\n",
					info.biXPelsPerMeter/39, info.biYPelsPerMeter/39,
					info.biClrUsed, info.biClrImportant);
	MessageBox(NULL, str, "Read BMP", MB_OK);

	if (info.biClrUsed > 0)
		n = info.biClrUsed;
	else
		n = 1<<info.biBitCount;

	bmp->cmap_n = n;
	bmp->cmap = (RGBQUAD *)malloc(sizeof(RGBQUAD) * (int)n);

	fread(bmp->cmap, sizeof(RGBQUAD), (int)n, bmp->file);

	if (info.biSizeImage == 0)
		info.biSizeImage = (header.bfSize - header.bfOffBits);

	n = info.biSizeImage/info.biHeight;

//	printf("scan line length: %d\n", n);

//	bmp->data = (char *)malloc(info.biHeight * (int)n);
	bmp->w = info.biWidth;
	bmp->h = info.biHeight;
	bmp->bw = n;

	return(bmp);
}

// reads, at most, max_n_lines of the bmp data into data
int ReadBmp(BMPinfo *bmp, char *data, long max_n_lines)
{
	int not_eof;
	long y;

	if (max_n_lines > bmp->h)
		max_n_lines = bmp->h;

	for(y = max_n_lines-1, not_eof = 1; y >= 0 && not_eof; y--)
		if (fread(&data[(int)y*bmp->bw], bmp->bw, 1, bmp->file) != 1)
			not_eof = 0;

	return(max_n_lines - (y + 1));
}

void CloseReadBmpFile(BMPinfo *bmp)
{
	fclose(bmp->file);
	free(bmp->cmap);
	free(bmp);
}

void WriteBmpHeader(BMPinfo *bmp, int n)
{
	BITMAPFILEHEADER header;
	BITMAPINFOHEADER info;

	// write the bmp header
	header.bfType = 'B' | ('M' << 8);
	header.bfOffBits = sizeof(header) + sizeof(info) +
										 bmp->cmap_n * sizeof(RGBQUAD);
	header.bfSize =  (header.bfOffBits +
										n * bmp->bw) / 4;
	header.bfReserved1 = header.bfReserved2 = 0;
	fwrite(&header, sizeof(header), 1, bmp->file);

//  printf("type:%d, size:%ld, offset:%ld\n",
//         header.bfType, header.bfSize, header.bfOffBits);
	// write thr bmp info
	info.biSize = sizeof(info);
	info.biWidth = bmp->bw;
	info.biHeight = n;
	info.biPlanes = 1;
	info.biBitCount = 8;
	info.biCompression = BI_RGB;
	info.biSizeImage = 0;   // 0 is OK if BI_RGB
	info.biXPelsPerMeter = 72*39L;
	info.biYPelsPerMeter = 72*39L;
	info.biClrUsed = 0;  // we use all the colors in the cmap
	info.biClrImportant = 0; // all colors are important
	fwrite(&info, sizeof(info), 1, bmp->file);

//  printf("w:%ld, h:%ld, planes:%d, bits:%d, comp:%ld, size:%ld\n",
//				 info.biWidth, info.biHeight, info.biPlanes, info.biBitCount,
//				 info.biCompression, info.biSizeImage);

//	printf("xres:%ld, yres:%ld, used:%ld, imp:%ld\n",
//					info.biXPelsPerMeter/39, info.biYPelsPerMeter/39,
//					info.biClrUsed, info.biClrImportant);

}

// open bmp file for write based on the info in bmp
BMPinfo *OpenWriteBmpFile(char *filename, BMPinfo *bmp)
{
	if ((bmp->file = fopen(filename, "wb")) == NULL)
		return(NULL);

	bmp->nLines = 0;

	// write header and info
	WriteBmpHeader(bmp, bmp->h);

	// write the CMAP
	fwrite(bmp->cmap, sizeof(RGBQUAD), (int)bmp->cmap_n, bmp->file);

	return(bmp);
}

// writes, at most, max_n_lines of the bmp data into data
int WriteBmp(BMPinfo *bmp, char *data, long max_n_lines)
{
	long y, n;
	int not_error;

	for(y = max_n_lines-1, not_error = 1; y >= 0 && not_error; y--)
		if (fwrite(&data[y*bmp->bw], bmp->bw, 1, bmp->file) != 1)
			not_error = 0;

	n = max_n_lines - (y + 1);
	bmp->nLines += n;
	return(n);
}

/* returns -1 if n < 0 : 1 if n >= 0 */

void CloseWriteBmpFile(BMPinfo *bmp)
{
	// if the bmp height turns out to be different then planed
	//    go back the begining and rewrite the header and info
	if (bmp->nLines != bmp->h) {
		rewind(bmp->file);
		WriteBmpHeader(bmp, bmp->nLines);
	}

	fclose(bmp->file);
}


