#define STRICT

#include <windows.h>
#include <windowsx.h>
#include <string.h>
#include <stdio.h>
#pragma hdrstop
#include <stdlib.h>
#include "defines.h"

static int nSampleRate = SAMPLERATE;
static int nCutoffFreq = CUTOFFFREQ;
static float fLinePeriod = LINETIME;
static int nLeftShift = 0;
static int nPassDir = PD_NORTH;
static int nLineDecimate = SCRNLINEDECIMATE;
static int nScrnDecimate = SCRNDECIMATE;
static int nFileDecimate = FILEDECIMATE;
static int nBrightness = SCRNBRIGHTNESS;
static float fContrast = SCRNCONTRAST;
static int bProcRT = 1;
static int bSaveWav = 0;

void WxLoadOptions(void)
{
	char *section = "Options";
	char *filename = "wx.ini";
	char str[80];
	char defstr[80];

	sprintf(defstr, "%f", WxGetLinePeriod());
	if (GetPrivateProfileString(section, "LinePeriod", defstr, str, 80, filename))
		WxSetLinePeriod(atof(str));

	sprintf(defstr, "%.1f", WxGetContrast());
	if (GetPrivateProfileString(section, "Contrast", defstr, str, 80, filename))
		WxSetContrast(atof(str));

	sprintf(defstr, "%.1d", WxGetBrightness());
	if (GetPrivateProfileString(section, "Brightness", defstr, str, 80, filename))
		WxSetBrightness(atoi(str));

	sprintf(defstr, "%d", WxGetLeftShift());
	if (GetPrivateProfileString(section, "LeftShift", defstr, str, 80, filename))
		WxSetLeftShift(atoi(str));

	sprintf(defstr, "%d", WxGetCutoffFreq());
	if (GetPrivateProfileString(section, "CutoffFreq", defstr, str, 80, filename))
		WxSetCutoffFreq(atoi(str));

	strcpy(defstr, WxGetPassDir() == PD_NORTH ? "north" : "south");
	if (GetPrivateProfileString(section, "PassDir", defstr, str, 80, filename))
		if (strcmp(str, "north") == 0)
			WxSetPassDir(PD_NORTH);
		else
			WxSetPassDir(PD_SOUTH);

	strcpy(defstr, WxGetProcRT() ? "yes" : "no");
	if (GetPrivateProfileString(section, "ProcessRealTime", defstr,
								str, 80, filename))
		if (strcmp(str, "yes") == 0)
			WxSetProcRT(1);
		else
			WxSetProcRT(0);
      
	strcpy(defstr, WxGetSaveWav() ? "yes" : "no");
	if (GetPrivateProfileString(section, "SaveWaveFile", defstr,
								str, 80, filename))
		if (strcmp(str, "yes") == 0)
			WxSetSaveWav(1);
		else
			WxSetSaveWav(0);

	sprintf(defstr, "%d", WxGetLineDecimate());
	if (GetPrivateProfileString(section, "LineDecimate", defstr, str,
				80, filename))
		WxSetLineDecimate(atoi(str));

	// do sample rate last becuase of the calc's it does
	sprintf(defstr, "%d", WxGetSampleRate());
	if (GetPrivateProfileString(section, "SampleRate", defstr, str, 80, filename))
		WxSetSampleRate(atoi(str));


}

void WxSaveOptions(void)
{
	char *section = "Options";
	char *filename = "wx.ini";
	char str[80];

	sprintf(str, "%f", WxGetLinePeriod());
	WritePrivateProfileString(section, "LinePeriod", str, filename);

	sprintf(str, "%.1f", WxGetContrast());
	WritePrivateProfileString(section, "Contrast", str, filename);

	sprintf(str, "%.1d", WxGetBrightness());
	WritePrivateProfileString(section, "Brightness", str, filename);

	sprintf(str, "%d", WxGetLeftShift());
	WritePrivateProfileString(section, "LeftShift", str, filename);

	sprintf(str, "%d", WxGetSampleRate());
	WritePrivateProfileString(section, "SampleRate", str, filename);

	sprintf(str, "%d", WxGetCutoffFreq());
	WritePrivateProfileString(section, "CutoffFreq", str, filename);

	strcpy(str, WxGetPassDir() == PD_NORTH ? "north" : "south");
	WritePrivateProfileString(section, "PassDir", str, filename);

	sprintf(str, "%d", WxGetLineDecimate());
	WritePrivateProfileString(section, "LineDecimate", str, filename);

	strcpy(str, WxGetProcRT() ? "yes" : "no");
	WritePrivateProfileString(section, "ProcessRealTime", str, filename);

	strcpy(str, WxGetSaveWav() ? "yes" : "no");
	WritePrivateProfileString(section, "SaveWaveFile", str, filename);
}

float WxGetLinePeriod(void)
{
	return(fLinePeriod);
}

float WxSetLinePeriod(float value)
{
	return(fLinePeriod = value);
}

float WxGetContrast(void)
{
	return(fContrast);
}

float WxSetContrast(float value)
{
	return(fContrast = value);
}

int WxGetBrightness(void)
{
	return(nBrightness);
}

int WxSetBrightness(int value)
{
	return(nBrightness = value);
}

int WxGetLeftShift(void)
{
	return(nLeftShift);
}

int WxSetLeftShift(int value)
{
	return(nLeftShift = value);
}

int WxGetPassDir(void)
{
	return(nPassDir);
}

int WxSetPassDir(int value)
{
	return(nPassDir = value);
}

int WxGetLineDecimate(void)
{
	return(nLineDecimate);
}

int WxSetLineDecimate(int value)
{
	return(nLineDecimate = value);
}

int WxGetScrnDecimate(void)
{
	return(nScrnDecimate);
}

int WxSetScrnDecimate(int value)
{
	return(nScrnDecimate = value);
}

int WxGetFileDecimate(void)
{
	return(nFileDecimate);
}

int WxSetFileDecimate(int value)
{
	return(nFileDecimate = value);
}

int WxGetSampleRate(void)
{
	return(nSampleRate);
}

int WxSetSampleRate(int value)
{
	nFileDecimate = value/4800;
	nScrnDecimate = nFileDecimate*2;
	int tmp = value/nFileDecimate;
	nSampleRate = tmp * nFileDecimate;
	return(nSampleRate);
}

int WxGetCutoffFreq(void)
{
	return(nCutoffFreq);
}

int WxSetCutoffFreq(int value)
{
	return(nCutoffFreq = value);
}

int WxGetProcRT(void)
{
	return(bProcRT);
}

int WxSetProcRT(int value)
{
	return(bProcRT = value);
}

int WxGetSaveWav(void)
{
	return(bSaveWav);
}

int WxSetSaveWav(int value)
{
	return(bSaveWav = value);
}

