#define STRICT

#include <windows.h>
#include <windowsx.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#pragma hdrstop
#include <mmsystem.h>
#include "wavfile.h"
#include "wxbmp.h"
#include "defines.h"

static WAVEFILE     WaveFile;
static LPWAVEFILE   lpWaveFile = &WaveFile;


inline
BYTE FullRectify(BYTE n)
{
	return(n < 127 ? 255 - n : n);
}

//inline
BYTE ByteMin(int n, BYTE m)
{
	return(n < m ? n : m);
}

inline
BYTE ByteClip(int n)
{
	if (n > 255)
		return(255);
	if (n < 0)
		return(0);
	return(n);
}



void ScanConv(LPBYTE lpBuf, int nBufSize)
{
	int NewSample();
//	const int nFilterSize = 4;
	int nDecimate = WxGetFileDecimate();
//	static BYTE bSamples[] = {128, 128, 128, 128};
//	static int nLastSample = 0;
//	static int nSum = nFilterSize * 128;
	static UINT nSum = 128;
	static int nCount = 0;
	int i;
	LPBYTE lpByte = lpBuf;
//	static nLowSum = 128;
	float K = 1.0 - exp(-2.0*WxGetCutoffFreq()*1.0/WxGetSampleRate());
	float L = 1.0 - K;
//	static int nLastAvg = 128;
//	static long nLineAvg = 0;

	for (i = 0; i < nBufSize; i++) {

//		nSum = FullRectify(*lpByte++);
//	  nSum -= bSamples[nLastSample];
//	  nSum += (bSamples[nLastSample] = FullRectify(*lpByte++));

		nSum = nSum * L + FullRectify(*lpByte++) * K;

//		nLineAvg += bSamples[nLastSample];

//		if (++nLastSample >= nFilterSize)
//			nLastSample = 0;

		if (++nCount >= nDecimate) {
			nCount = 0;
			// // input samples are 128-255 and four are summed
			// // so divide by 4 (>> 2) - 128 (now (0-128))
			// // multiple by 2 (<< 1) (now its (0 - 255)
			// typical numbers are 143 to 199, (nSum-143)*255/(199-143)
			AddBitmapPixel(ByteClip((nSum - (255 - WxGetBrightness())) * WxGetContrast()));


//			AddBitmapPixel(ByteClip((nLastAvg - 128) << 1));
//			AddBitmapPixel(ByteClip(((nSum) - 128) << 1));
//			AddBitmapPixel(ByteClip((nSum - 128) << 1));

		}
		NewSample();   // count input samples
	}

}

// return 0 - OK  1 - failed
int DoWavFile(char *szFileName, HWND hwndApp)
{
	const int nBufSize = 9600;   // this is arbitrary and is not the samplerate
	lpWaveFile->hwndApp = hwndApp;

	if (openReadWaveFile(szFileName, lpWaveFile)) return(1);
	WxSetSampleRate(lpWaveFile->pFormat->nSamplesPerSec);
	int ls = WxGetLeftShift()*WxGetSampleRate()*WxGetLinePeriod()/100;
	int n = (lpWaveFile->dwDataSize-ls) / nBufSize;
	int nRemainder = lpWaveFile->dwDataSize % nBufSize;
	lpWaveFile->dwDataSize = nBufSize;
	if (allocWaveData(lpWaveFile)) return(1);

	InitBitmap(hwndApp);
	InitWxFileBitmap(100);

	// Skip the left shift amount (which is 0 -> 100)
	lpWaveFile->dwDataSize = ls;
	if (readWaveFileData(lpWaveFile)) return(1);

	lpWaveFile->dwDataSize = nBufSize;
	for (int i = 0; i < n; i++) {
		if (readWaveFileData(lpWaveFile)) return(1);
		ScanConv(lpWaveFile->lpData, lpWaveFile->dwDataSize);
/*
		MSG Msg;
		GetMessage(&Msg, NULL, 0, 0);
		TranslateMessage(&Msg);
		DispatchMessage(&Msg);
*/
	}
	/* read the last one */
	lpWaveFile->dwDataSize = nRemainder;
	if (readWaveFileData(lpWaveFile)) return(1);
	ScanConv(lpWaveFile->lpData, lpWaveFile->dwDataSize);

	if (closeWaveFile(lpWaveFile)) return(1);

	closeWxFileBitmap();

	return(0);
}


