/*
    sound.c

*/
#define STRICT

#include <windows.h>
#include <windowsx.h>
#include <string.h>
#include <stdio.h>
#pragma hdrstop
#include <mmsystem.h>
#include "sound.h"


//
// local data
//

static BOOL bInputActive = 0;
static HWAVEIN hWaveIn;
static HWAVEOUT hWaveOut;
static BOOL bAborted;
static LPWAVEHDR   lpWaveHdr;
static HWND hwndApp;

static nBufferSize;
#define NUMBUFFERS 3
// NULL at least the first one
static LPWAVEHDR alpBufferHdrs[NUMBUFFERS] = {NULL};

/******** Wave Device interface *********/
/* wave output */

int openOutWaveDevice(HWND hwndMain, int nSampleRate, int nBits, int nChannels)
{
		PCMWAVEFORMAT wf;

		hwndApp = hwndMain;

		//
		// Open the wave in device
		//

		wf.wBitsPerSample = nBits;
		wf.wf.wFormatTag = WAVE_FORMAT_PCM;
		wf.wf.nChannels = nChannels;
		wf.wf.nSamplesPerSec = nSampleRate;
		wf.wf.nAvgBytesPerSec = nSampleRate;
		wf.wf.nBlockAlign = nBits/8;

	/* Make sure a waveform output device supports this format.
	 */
	if (waveOutOpen(&hWaveOut, (UINT)WAVE_MAPPER,
			(LPWAVEFORMAT)&wf, NULL, 0L,
			(DWORD)WAVE_FORMAT_QUERY))
		{
			MessageBox(hwndApp,
				 "The waveform device can't play this format.",
					NULL, MB_OK | MB_ICONEXCLAMATION);
			return(-1);
		}


	/* Open a waveform output device.
	 */
	if (waveOutOpen(&hWaveOut, (UINT)WAVE_MAPPER,
			(LPWAVEFORMAT)&wf,
			(UINT)hwndApp, 0L, (DWORD)CALLBACK_WINDOW))
		{
			MessageBox(hwndApp,
					"Failed to open waveform output device.",
					NULL, MB_OK | MB_ICONEXCLAMATION);
			return(-1);
		}

	return(0);
}

/* hData must have GlobalAlloc'd and GloabalLock'd */
int playWaveDevice(HANDLE hData, LPBYTE lpData, DWORD dwDataSize)
{
	HANDLE          hWaveHdr;
	LPWAVEINST      lpWaveInst;
	WORD            wResult;
	HANDLE          hWaveInst;

	/* Allocate a waveform data header. The WAVEHDR must be
	 * globally allocated and locked.
	 */
	hWaveHdr = GlobalAlloc(GMEM_MOVEABLE | GMEM_SHARE,
			 (DWORD) sizeof(WAVEHDR));
	if (!hWaveHdr)
		{
			GlobalUnlock( hData );
			GlobalFree( hData );
			MessageBox(hwndApp, "Not enough memory for header.",
					NULL, MB_OK | MB_ICONEXCLAMATION);
			return(-1);
		}

	lpWaveHdr = (LPWAVEHDR) GlobalLock(hWaveHdr);
	if (!lpWaveHdr)
		{
			GlobalUnlock( hData );
			GlobalFree( hData );
			GlobalFree( hWaveHdr );
			MessageBox(hwndApp, "Failed to lock memory for header.",
					 NULL, MB_OK | MB_ICONEXCLAMATION);
			return(-1);
		}

	/* Allocate and set up instance data for waveform data block.
	 * This rmation is needed by the routine that frees the
	 * data block after it has been played.
	 */
	hWaveInst = GlobalAlloc(GMEM_MOVEABLE | GMEM_SHARE,
				(DWORD) sizeof(WAVEHDR));
	if (!hWaveInst)
		{
			GlobalUnlock( hData );
			GlobalFree( hData );
			GlobalUnlock( hWaveHdr );
			GlobalFree( hWaveHdr );
			MessageBox(hwndApp,
				 "Not enough memory for instance data.",
					NULL, MB_OK | MB_ICONEXCLAMATION);
			return(-1);
		}

	lpWaveInst = (LPWAVEINST) GlobalLock(hWaveInst);
	if (!lpWaveInst)
		{
			GlobalUnlock( hData );
			GlobalFree( hData );
			GlobalUnlock( hWaveHdr );
			GlobalFree( hWaveHdr );
			GlobalFree( hWaveInst );
			MessageBox(hwndApp,
						"Failed to lock memory for instance data.",
						NULL, MB_OK | MB_ICONEXCLAMATION);
			return(-1);
		}

	lpWaveInst->hWaveInst = hWaveInst;
	lpWaveInst->hWaveHdr = hWaveHdr;
	lpWaveInst->hWaveData = hData;

	/* Set up WAVEHDR structure and prepare it to be written to wave device.
	 */
	lpWaveHdr->lpData = (char *)lpData;
	lpWaveHdr->dwBufferLength = dwDataSize;
  lpWaveHdr->dwFlags = 0L;
	lpWaveHdr->dwLoops = 0L;
	lpWaveHdr->dwUser = (DWORD) lpWaveInst;
	if(waveOutPrepareHeader(hWaveOut, lpWaveHdr, sizeof(WAVEHDR)))
    {
			GlobalUnlock( hData );
			GlobalFree( hData );
			GlobalUnlock( hWaveHdr );
			GlobalFree( hWaveHdr );
			GlobalUnlock( hWaveInst );
      GlobalFree( hWaveInst );
			MessageBox(hwndApp, "Unable to prepare wave header.",
					NULL, MB_OK | MB_ICONEXCLAMATION);

			return(-1);
		}

	/* Then the data block can be sent to the output device.
   */
	wResult = waveOutWrite(hWaveOut, lpWaveHdr, sizeof(WAVEHDR));
  if (wResult != 0)
		{
			waveOutUnprepareHeader(hWaveOut, lpWaveHdr, sizeof(WAVEHDR));
			GlobalUnlock( hData );
			GlobalFree( hData );
			MessageBox(hwndApp, "Failed to write block to device",
				 NULL, MB_OK | MB_ICONEXCLAMATION);
			return(-1);
		}

	return(0);
}

void cleanupWaveDevice(LPWAVEINST lpWaveInst)
{
//	GlobalUnlock( lpWaveInst->hWaveData );
//	GlobalFree( lpWaveInst->hWaveData );
	GlobalUnlock( lpWaveInst->hWaveHdr );
	GlobalFree( lpWaveInst->hWaveHdr );
	GlobalUnlock( lpWaveInst->hWaveInst );
	GlobalFree( lpWaveInst->hWaveInst );
}


void closeOutWaveDevice(void)
{
	LPWAVEINST lpWaveInst;

	if (hWaveOut)
		{
			waveOutReset(hWaveOut);
			waveOutUnprepareHeader(hWaveOut, lpWaveHdr, sizeof(WAVEHDR));
			lpWaveInst = (LPWAVEINST)(lpWaveHdr->dwUser);
			cleanupWaveDevice(lpWaveInst);
			waveOutClose(hWaveOut);
		}
}



/* WAV input */

/* Record opens the device and starts recording
	 the MW_WIM_DATA is posted as each buffer is filled */

void recordWaveDevice(HWND hwndMain, int nSampleRate, int nBits,
						int nChannels, int nBufSize)
{
		PCMWAVEFORMAT wf;
		WORD wResult;

		nBufferSize = nBufSize;

		//
		// Open the wave in device
		//

		wf.wBitsPerSample = nBits;
		wf.wf.wFormatTag = WAVE_FORMAT_PCM;
		wf.wf.nChannels = nChannels;
		wf.wf.nSamplesPerSec = nSampleRate;
		wf.wf.nAvgBytesPerSec = nSampleRate;
		wf.wf.nBlockAlign = nBits/8;

		wResult = waveInOpen(&hWaveIn,
												 (UINT)WAVE_MAPPER,
												 (LPWAVEFORMAT) &wf,
												 (DWORD) MAKELONG(hwndMain, 0),
												 0,
												 CALLBACK_WINDOW);

		if (wResult == 0) {

				//
				// Add two buffers to the input queue and begin recording
				//

				bInputActive = TRUE;
				bAborted = FALSE;
				if (addInWaveDeviceBuffer()) {
						if (addInWaveDeviceBuffer()) {
								waveInStart(hWaveIn);
								//
								// Put up a modal window and just process data blocks
								// until user hits the cancel button
								//
					 //			MessageBox(hwndMain, "Hit OK to end", "Recording!", MB_OK);
						//		closeInWaveDevice();
						}
				}
		} else {
				hWaveIn = NULL;
		}
}


void closeInWaveDevice(void)
{
		if (!hWaveIn) return;

		bInputActive = FALSE;

		waveInStop(hWaveIn);
		waveInReset(hWaveIn);
		waveInClose(hWaveIn);

		hWaveIn = NULL;
}

//
// add an input buffer to the wave in device queue
//

BOOL addInWaveDeviceBuffer(void)
{
		HANDLE hHdr, hData;
		LPWAVEHDR lpHdr;
		LPSTR lpData;
		WORD wResult;
		int i;
		static int nNextBuffer = 0;

		// allocate memory for the header and the data buffer

		if (alpBufferHdrs[0] == NULL) {
			for (i = 0; i < NUMBUFFERS; i++) {
				hHdr = GlobalAlloc(GMEM_MOVEABLE | GMEM_SHARE, sizeof(WAVEHDR));
				hData = GlobalAlloc(GMEM_MOVEABLE| GMEM_SHARE, nBufferSize);
				if (!hHdr || !hData)
					return FALSE;
				lpHdr = (LPWAVEHDR) GlobalLock(hHdr);
				lpData = (LPSTR)GlobalLock(hData);

				lpHdr->lpData = lpData;
				lpHdr->dwBufferLength = nBufferSize;
				lpHdr->dwUser = 0;
				lpHdr->dwFlags = 0;

				wResult = waveInPrepareHeader(hWaveIn, lpHdr, sizeof(WAVEHDR));
				if (wResult != 0)
					return FALSE;

				alpBufferHdrs[i] = lpHdr;
			}
		}


		wResult = waveInAddBuffer(hWaveIn, alpBufferHdrs[nNextBuffer],
															sizeof(WAVEHDR));
		if (wResult != 0)
				return FALSE;

		if (++nNextBuffer >= NUMBUFFERS)
			nNextBuffer = 0;

		return TRUE;
}

BOOL waveInputActive(void)
{
	return(bInputActive);
}

//
// Add the another buffer
//
#pragma argsused
void continueWaveInput(HWAVEIN hDev, LPWAVEHDR lpHdr)
{
		HANDLE hMem;
		int i;

		// if we are still active,
		// add a new buffer to the driver.
		if (bInputActive) {

				//
				// Add a new buffer to the driver list
				//
				if (!addInWaveDeviceBuffer()) {
					closeInWaveDevice();
				}

		} else {
				// this is called when the user stops recording
				// and we get your last WM_OIM_DATA
				// If input not active
				// Tidy up the buffers
				//
				for (i = 0; i < NUMBUFFERS; i++) {
					hMem = (HANDLE) HIWORD(alpBufferHdrs[i]->lpData);
					GlobalUnlock(hMem);
					GlobalFree(hMem);

					waveInUnprepareHeader(hDev, alpBufferHdrs[i], sizeof(WAVEHDR));

					hMem = (HANDLE) HIWORD(alpBufferHdrs[i]);
					GlobalUnlock(hMem);
					GlobalFree(hMem);
			}
		}

}



