#define STRICT

#include <windows.h>
#include <windowsx.h>
#include <string.h>
#include <stdio.h>
#pragma hdrstop
#include <mmsystem.h>
#include "wavfile.h"

/* open an existing wave file */
/* return -1 on failure, 0 on success */
/* lpWaveFile should be blank EXPECT for hwnd */
int openReadWaveFile(char *szFileName, LPWAVEFILE lpWaveFile)
{

  HMMIO           hmmio;
  MMCKINFO        mmckinfoParent;
  MMCKINFO        mmckinfoSubchunk;
  DWORD           dwFmtSize;
  DWORD		        dwDataSize;
  HANDLE          hFormat;
  WAVEFORMAT      *pFormat;
	HWND            hwndApp = lpWaveFile->hwndApp;

	/* check for valid hwnd */
	if (!hwndApp) {
		MessageBox(NULL, "The hwndApp in lpWaveFile not valid.",
				 NULL, MB_OK | MB_ICONEXCLAMATION);
		return(-1);
	}

	/* Open the given file for reading using buffered I/O.
   */
	if(!(hmmio = mmioOpen(szFileName, NULL, MMIO_READ | MMIO_ALLOCBUF)))
		{
      MessageBox(hwndApp, "Failed to open file.",
		 NULL, MB_OK | MB_ICONEXCLAMATION);
      return(-1);
    }

  /* Locate a 'RIFF' chunk with a 'WAVE' form type
   * to make sure it's a WAVE file.
	 */
  mmckinfoParent.fccType = mmioFOURCC('W', 'A', 'V', 'E');
  if (mmioDescend(hmmio, (LPMMCKINFO) &mmckinfoParent, NULL, MMIO_FINDRIFF))
		{
			MessageBox(hwndApp, "This is not a WAVE file.",
								 NULL, MB_OK | MB_ICONEXCLAMATION);
			mmioClose(hmmio, 0);
			return(-1);
		}

  /* Now, find the format chunk (form type 'fmt '). It should be
   * a subchunk of the 'RIFF' parent chunk.
   */
  mmckinfoSubchunk.ckid = mmioFOURCC('f', 'm', 't', ' ');
  if (mmioDescend(hmmio, &mmckinfoSubchunk, &mmckinfoParent, 
		  MMIO_FINDCHUNK))
    {
      MessageBox(hwndApp, "WAVE file is corrupted.",
		 NULL, MB_OK | MB_ICONEXCLAMATION);
      mmioClose(hmmio, 0);
      return(-1);
    }

  /* Get the size of the format chunk, allocate and lock memory for it.
   */
  dwFmtSize = mmckinfoSubchunk.cksize;
  hFormat = LocalAlloc(LMEM_MOVEABLE, LOWORD(dwFmtSize));
  if (!hFormat)
    {
      MessageBox(hwndApp, "Out of memory.",
		 NULL, MB_OK | MB_ICONEXCLAMATION);
      mmioClose(hmmio, 0);
      return(-1);
    }
  pFormat = (WAVEFORMAT *) LocalLock(hFormat);
  if (!pFormat)
    {
      MessageBox(hwndApp, "Failed to lock memory for format chunk.",
		 NULL, MB_OK | MB_ICONEXCLAMATION);
      LocalFree( hFormat );
      mmioClose(hmmio, 0);
      return(-1);
    }

  /* Read the format chunk.
   */
  if (mmioRead(hmmio, (HPSTR) pFormat, dwFmtSize) != (LONG) dwFmtSize)
    {
      MessageBox(hwndApp, "Failed to read format chunk.",
		 NULL, MB_OK | MB_ICONEXCLAMATION);
      LocalUnlock( hFormat );
      LocalFree( hFormat );
      mmioClose(hmmio, 0);
      return(-1);
    }
    
  /* Make sure it's a PCM file.
   */
  if (pFormat->wFormatTag != WAVE_FORMAT_PCM)
    {
      LocalUnlock( hFormat );
      LocalFree( hFormat );
      mmioClose(hmmio, 0);
      MessageBox(hwndApp, "The file is not a PCM file.",
		 NULL, MB_OK | MB_ICONEXCLAMATION);
      return(-1);
    }

  /* Ascend out of the format subchunk.
   */
  mmioAscend(hmmio, &mmckinfoSubchunk, 0);
    
  /* Find the data subchunk.
   */
  mmckinfoSubchunk.ckid = mmioFOURCC('d', 'a', 't', 'a');
  if (mmioDescend(hmmio, &mmckinfoSubchunk, &mmckinfoParent,
		  MMIO_FINDCHUNK))
    {
      MessageBox(hwndApp, "WAVE file has no data chunk.",
		 NULL, MB_OK | MB_ICONEXCLAMATION);
      LocalUnlock( hFormat );
      LocalFree( hFormat );
      mmioClose(hmmio, 0);
      return(-1);
    }

  /* Get the size of the data subchunk.
   */
  dwDataSize = mmckinfoSubchunk.cksize;
  if (dwDataSize == 0L)
    {
      MessageBox(hwndApp, "The data chunk has no data.",
		 NULL, MB_OK | MB_ICONEXCLAMATION);
      LocalUnlock( hFormat );
      LocalFree( hFormat );
      mmioClose(hmmio, 0);
      return(-1);
    }

  lpWaveFile->hmmio = hmmio;
  lpWaveFile->dwDataSize = dwDataSize;
  lpWaveFile->hFormat = hFormat;
  lpWaveFile->pFormat = pFormat;

  return(0);
}

/* dwDataSize must be valid */
int allocWaveData(LPWAVEFILE lpWaveFile)
{
	/* Allocate and lock memory for the waveform data.
	 */
	lpWaveFile->hData = GlobalAlloc(GMEM_MOVEABLE | GMEM_SHARE,
					lpWaveFile->dwDataSize );
	if (!lpWaveFile->hData)
		{
			MessageBox(lpWaveFile->hwndApp, "Out of memory.",
									NULL, MB_OK | MB_ICONEXCLAMATION);
			if (lpWaveFile->hmmio)
				mmioClose(lpWaveFile->hmmio, 0);
			return(-1);
		}
	lpWaveFile->lpData = (LPBYTE)GlobalLock(lpWaveFile->hData);
	if (!lpWaveFile->lpData)
		{
      MessageBox(lpWaveFile->hwndApp, "Failed to lock memory for data chunk.",
		 NULL, MB_OK | MB_ICONEXCLAMATION);
			GlobalFree( lpWaveFile->hData );
			if (lpWaveFile->hmmio)
				 mmioClose(lpWaveFile->hmmio, 0);
			return(-1);
		}

	return(0);
}

int readWaveFileData(LPWAVEFILE lpWaveFile)
{
	/* Read the waveform data subchunk.
	 */
	if(mmioRead(lpWaveFile->hmmio, (HPSTR) lpWaveFile->lpData,
				lpWaveFile->dwDataSize) != (LONG) lpWaveFile->dwDataSize)
		{
			MessageBox(lpWaveFile->hwndApp, "Failed to read data chunk.",
								 NULL, MB_OK | MB_ICONEXCLAMATION);
			GlobalUnlock( lpWaveFile->hData );
			GlobalFree( lpWaveFile->hData );
			mmioClose(lpWaveFile->hmmio, 0);
			return(-1);
		}
	return(0);
}

int closeWaveFile(LPWAVEFILE lpWaveFile)
{
	/* We're done with the file, close it.
	 */
	if (lpWaveFile->hmmio != NULL)
	 	mmioClose(lpWaveFile->hmmio, 0);

	return(0);
}



/* output WAVE file */
/* return -1 on failure, 0 on success */
/* lpWaveFile->hwndApp and pFormat must be valid */
int openWriteWaveFile(char *szFileName, LPWAVEFILE lpWaveFile)
{
		HMMIO       hmmioOut = NULL;    // handle to open output WAVE file
		MMCKINFO    ckOutRIFF;          // chunk info. for output RIFF chunk
		MMCKINFO    ckOut;              // info. for a chunk in output file
		HWND        hwndApp = lpWaveFile->hwndApp;

	/* check for valid hwnd */
	if (!hwndApp) {
		MessageBox(NULL, "The hwndApp in lpWaveFile not valid.",
				 NULL, MB_OK | MB_ICONEXCLAMATION);
		return(-1);
	}

		/* Open the output file for writing using buffered I/O. Note that
		 * if the file exists, the MMIO_CREATE flag causes it to be truncated
		 * to zero length.
		 */
		if (!(hmmioOut = mmioOpen(szFileName, NULL,
												MMIO_ALLOCBUF | MMIO_WRITE | MMIO_CREATE))) {
			MessageBox(hwndApp, "Failed to open file.",
								 NULL, MB_OK | MB_ICONEXCLAMATION);
      return(-1);
		}

		lpWaveFile->hmmio = hmmioOut;

		/* Create the output file RIFF chunk of form type 'WAVE'.
		 */
		ckOutRIFF.fccType = mmioFOURCC('W', 'A', 'V', 'E');
		if (mmioCreateChunk(hmmioOut, &ckOutRIFF, MMIO_CREATERIFF) != 0)
		{
			MessageBox(hwndApp, "Can't create WAVE chunk",
								 NULL, MB_OK | MB_ICONEXCLAMATION);
			mmioClose(hmmioOut, 0);
			return(-1);
		}

		lpWaveFile->ckOutRIFF = ckOutRIFF;

		/* We are now descended into the 'RIFF' chunk we just created.
		 * Now create the 'fmt ' chunk. Since we know the size of this chunk,
		 * specify it in the MMCKINFO structure so MMIO doesn't have to seek
		 * back and set the chunk size after ascending from the chunk.
		 */
		ckOut.ckid = mmioFOURCC('f', 'm', 't', ' ');
		ckOut.cksize = sizeof(PCMWAVEFORMAT);   // we know the size of this ck.
		if (mmioCreateChunk(hmmioOut, &ckOut, 0) != 0)
		{
			MessageBox(hwndApp, "Can't create FMT chunk",
								 NULL, MB_OK | MB_ICONEXCLAMATION);
			mmioClose(hmmioOut, 0);
			return(-1);
		}


		/* Write the PCMWAVEFORMAT structure to the 'fmt ' chunk.
		 */
		if (mmioWrite(hmmioOut, (HPSTR)lpWaveFile->pFormat, sizeof(PCMWAVEFORMAT))
				!= sizeof(PCMWAVEFORMAT))
		{
			MessageBox(hwndApp, "Can't write the format chunk",
								 NULL, MB_OK | MB_ICONEXCLAMATION);
			mmioClose(hmmioOut, 0);
			return(-1);
		}

		/* Ascend out of the 'fmt ' chunk, back into the 'RIFF' chunk.
		 */
		if (mmioAscend(hmmioOut, &ckOut, 0) != 0)
		{
			MessageBox(hwndApp, "Can't ascend from fmt chunk",
								 NULL, MB_OK | MB_ICONEXCLAMATION);
			mmioClose(hmmioOut, 0);
			return(-1);
		}

		/* Create the 'data' chunk that holds the waveform samples.
		 */
		ckOut.ckid = mmioFOURCC('d', 'a', 't', 'a');
		if (mmioCreateChunk(hmmioOut, &ckOut, 0) != 0)
		{
			MessageBox(hwndApp, "Can't create data chunk",
								 NULL, MB_OK | MB_ICONEXCLAMATION);
			mmioClose(hmmioOut, 0);
			return(-1);
		}

	lpWaveFile->ckOut = ckOut;
	return(0);  /* file is ready for data */
}

int writeWaveFileData(LPWAVEFILE lpWaveFile)
{
	/* Write the waveform data subchunk.
	 */
	if(mmioWrite(lpWaveFile->hmmio, (HPSTR) lpWaveFile->lpData,
				lpWaveFile->dwDataSize) != (LONG) lpWaveFile->dwDataSize)
		{
			MessageBox(lpWaveFile->hwndApp, "Failed to read data chunk.",
				 NULL, MB_OK | MB_ICONEXCLAMATION);
			GlobalUnlock( lpWaveFile->hData );
			GlobalFree( lpWaveFile->hData );
			mmioClose(lpWaveFile->hmmio, 0);
			return(-1);
		}
	return(0);
}

int closeWriteWaveFile(LPWAVEFILE lpWaveFile)
{
		/* Ascend the output file out of the 'data' chunk -- this will cause
		 * the chunk size of the 'data' chunk to be written.
		 */
		if (mmioAscend(lpWaveFile->hmmio, &lpWaveFile->ckOut, 0) != 0)
		{
			MessageBox(lpWaveFile->hwndApp, "Failed to read data chunk.",
								 NULL, MB_OK | MB_ICONEXCLAMATION);
			GlobalUnlock( lpWaveFile->hData );
			GlobalFree( lpWaveFile->hData );
			mmioClose(lpWaveFile->hmmio, 0);
			return(-1);
		}

		/* Ascend the output file out of the 'RIFF' chunk -- this will cause
		 * the chunk size of the 'RIFF' chunk to be written.
		 */
		if (mmioAscend(lpWaveFile->hmmio, &lpWaveFile->ckOutRIFF, 0) != 0)
		{
			MessageBox(lpWaveFile->hwndApp, "Failed to write data chunk.",
								 NULL, MB_OK | MB_ICONEXCLAMATION);
			GlobalUnlock( lpWaveFile->hData );
			GlobalFree( lpWaveFile->hData );
			mmioClose(lpWaveFile->hmmio, 0);
			return(-1);
		}
		/* Close the file(unless they weren't opened successfully).
		 */
		if (lpWaveFile->hmmio != NULL)
				mmioClose(lpWaveFile->hmmio, 0);

		return(0);
}


