// =====================================================================
// WX.CPP : Weather Satellite program for Windows 3.1
// =====================================================================
#define STRICT

#include <windows.h>
#include <windowsx.h>
#include <string.h>
#include <stdio.h>
#pragma hdrstop
#include <stdlib.h>
#include <mmsystem.h>
#include "wx.h"
#include "wxbmp.h"
#include "bmp.h"
#include "sound.h"
#include "wavfile.h"
#include "cdialog.h"
#include "defines.h"

#pragma warning (disable: 4068)
#pragma warning (disable: 4100)

// Globals
static char szAppName[] = "Wx";
static HINSTANCE hInstance;
static HBITMAP TheBitmap;

static HWND MainWindow;

WAVEFILE     WaveFile;
LPWAVEFILE   lpWaveFile = &WaveFile;


// ====================================================================
// INITIALIZATION
// ====================================================================

/////////////////////////////////////////
// Window application entry point (main)
/////////////////////////////////////////

#pragma argsused
int PASCAL WinMain(HINSTANCE hInst, HINSTANCE hPrevInstance,
						 LPSTR lpszCmdParam, int nCmdShow)
{
  MSG Msg;

  // Register the program if not already opened
  if (!hPrevInstance)
	 if (!Register(hInst))
		return FALSE;

  // Create the programs MainWindow
  if ((MainWindow = Create(hInst, nCmdShow)) == FALSE)
	 return FALSE;

  // Enter Event Loop
  while (GetMessage(&Msg, NULL, 0, 0))
  {
	 TranslateMessage(&Msg);
	 DispatchMessage(&Msg);
  }

  return Msg.wParam;
}


///////////////////////////////////////////////////
// Register the window
//////////////////////////////////////////////////
BOOL Register(HINSTANCE hInst)
{
  WNDCLASS WndClass;

  WndClass.style         = CS_HREDRAW | CS_VREDRAW;
  WndClass.lpfnWndProc   = WndProc;
  WndClass.cbClsExtra    = 0;
  WndClass.cbWndExtra    = 0;
  WndClass.hInstance     = hInst;
  WndClass.hIcon         = LoadIcon(hInst, "Icon");
  WndClass.hCursor       = LoadCursor(hInst, "Cursor");
  WndClass.hbrBackground = GetStockBrush(WHITE_BRUSH);
  WndClass.lpszMenuName  = "Menu";
  WndClass.lpszClassName = szAppName;

  return RegisterClass(&WndClass);
}

///////////////////////////////////////////////////
// Create the window
//////////////////////////////////////////////////
HWND Create(HINSTANCE hInst, int nCmdShow)
{
  hInstance = hInst;

  HWND hWindow = CreateWindow(szAppName, szAppName,
										WS_OVERLAPPEDWINDOW | WS_HSCROLL | WS_VSCROLL,
										CW_USEDEFAULT, CW_USEDEFAULT,
										CW_USEDEFAULT, CW_USEDEFAULT,
										NULL, NULL, hInst, NULL);

  if (hWindow != NULL) {
	 ShowWindow(hWindow, nCmdShow);
	 UpdateWindow(hWindow);
  }

  return hWindow;
}

// ====================================================================
// IMPLEMENT WINDOW BEHAVIOR
// ====================================================================

/* void fn(HWND hwnd, HWAVEIN hWaveIn, LPWAVEHDR lpWaveHdr)
		call for data received from the WAVE input device */
#define HANDLE_MM_WIM_DATA(hwnd, wParam, lParam, fn) \
		((fn)((hwnd), (HWAVEIN)(wParam), (LPWAVEHDR)(lParam)), 0L)

/* void fn(HWND hwnd)
		call for data received from the WAVE input device */
#define HANDLE_MM_WIM_CLOSE(hwnd, wParam, lParam, fn) \
		((fn)(hwnd), 0L)

////////////////////////////////////////////////////////////
// The Window proc is where the messages get processed
////////////////////////////////////////////////////////////

LRESULT CALLBACK _export WndProc(HWND hWindow, UINT Message,
											 WPARAM wParam, LPARAM lParam)
{
	switch (Message) {
	 HANDLE_MSG(hWindow, WM_CREATE, Wx_OnCreate);
	 HANDLE_MSG(hWindow, WM_DESTROY, Wx_OnDestroy);
	 HANDLE_MSG(hWindow, WM_COMMAND, Wx_OnCommand);
	 HANDLE_MSG(hWindow, WM_PAINT, Wx_OnPaint);
	 HANDLE_MSG(hWindow, WM_HSCROLL, Wx_OnHScroll);
	 HANDLE_MSG(hWindow, WM_VSCROLL, Wx_OnVScroll);
	 HANDLE_MSG(hWindow, MM_WIM_DATA, Wx_OnWim_Data);
	 HANDLE_MSG(hWindow, MM_WIM_CLOSE, Wx_OnWim_Close);
	 default : return Wx_DefProc(hWindow, Message, wParam, lParam);
	}
}

void ArrangeWindows(HWND hwnd)
{
	RECT Rect;
	GetClientRect(hwnd, &Rect);
}

////////////////////////////////////////////////////////////
// Handle the WM_CREATE message
////////////////////////////////////////////////////////////
#pragma argsused
BOOL Wx_OnCreate(HWND hwnd, CREATESTRUCT FAR* lpCreateStruct)
{

	RECT Rect;

	if ((TheBitmap = LoadBitmap(hInstance, "Bitmap")) == NULL) {
		MessageBox(hwnd, "No Bitmap", "Fatal Error", MB_ICONSTOP);
		return FALSE;
	}

  ArrangeWindows(hwnd);

	GetClientRect(hwnd, &Rect);
	SetScrollRange(hwnd, SB_HORZ, 0,
								SCRNBMPWIDTH-(Rect.right-Rect.left), FALSE);

	WxLoadOptions();

	return TRUE;
}

////////////////////////////////////////////////////////////
// Handle the WM_DESTORY message
////////////////////////////////////////////////////////////
#pragma argsused
void Wx_OnDestroy(HWND hwnd)
{
	void DestroyBitmap(void);

	DeleteBitmap(TheBitmap);
	DestroyBitmap();
	PostQuitMessage(0);
}

////////////////////////////////////////////////////////////
// Handle the WM_COMMAND message
////////////////////////////////////////////////////////////
#pragma argsused
void Wx_OnCommand(HWND hwnd, int id,
								HWND hwndCtl, UINT codeNotify)
{
	void ReadBitmap(HWND hwnd);
	void RealTime(HWND hwnd);
	void StopRealTime(HWND hwnd);
	void PostProcess(HWND hwnd);
	void SaveAs(HWND hwnd);
	BOOL CALLBACK _export AboutDlgProc(HWND hDlg, UINT Message,
													WPARAM wParam, LPARAM lParam);
	BOOL CALLBACK _export OptionsDlgProc(HWND hDlg, UINT Message,
													WPARAM wParam, LPARAM lParam);

	switch (id) {
	case CM_ABOUT : {
		FARPROC AboutBox = MakeProcInstance((FARPROC)AboutDlgProc, hInstance);
		DialogBox(hInstance, "AboutDialog", hwnd, (DLGPROC)AboutBox);
		FreeProcInstance(AboutBox);
		break;
	}
	case CM_OPTIONS : {
		FARPROC OptionsBox = MakeProcInstance((FARPROC)OptionsDlgProc, hInstance);
		DialogBox(hInstance, "OptionsDialog", hwnd, (DLGPROC)OptionsBox);
		FreeProcInstance(OptionsBox);
		break;
	}
	case CM_REALTIME: RealTime(hwnd); break;
	case CM_POSTPROCESS: PostProcess(hwnd); break;
	case CM_DISPLAYBMP: ReadBitmap(hwnd); break;
	case CM_STOP_REALTIME: StopRealTime(hwnd); break;
	case CM_SAVE_AS: SaveAs(hwnd); break;
	case CM_EXIT: PostMessage(hwnd, WM_DESTROY, 0, 0L); break;
	}
}


void NotYet(HWND hwnd)
{
	MessageBox(hwnd, "NOT YET!!", "Under Construction",
					 MB_ICONEXCLAMATION);
}

void Message(HWND hwnd, char *szString)
{
	MessageBox(hwnd, szString, "Wx",
					 MB_ICONEXCLAMATION);
}

#pragma argsused
BOOL CALLBACK _export AboutDlgProc(HWND hDlg, UINT Message,
												WPARAM wParam, LPARAM lParam)
{
	switch (Message) {
	case WM_INITDIALOG: return TRUE;
	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK ||
			 LOWORD(wParam) == IDCANCEL) {
				 EndDialog(hDlg, IDOK);
				 return TRUE;
		}
		break;
	}
	return FALSE;
}

#pragma argsused
BOOL CALLBACK _export OptionsDlgProc(HWND hDlg, UINT Message,
												WPARAM wParam, LPARAM lParam)
{
	switch (Message) {
	case WM_INITDIALOG:
		char str[16];
		sprintf(str, "%f", WxGetLinePeriod());
		SendDlgItemMessage(hDlg, IDC_LINEPERIOD, WM_SETTEXT, 0, (long)str);
		sprintf(str, "%.1f", WxGetContrast());
		SendDlgItemMessage(hDlg, IDC_CONTRAST, WM_SETTEXT, 0, (long)str);
		sprintf(str, "%.1d", WxGetBrightness());
		SendDlgItemMessage(hDlg, IDC_BRIGHTNESS, WM_SETTEXT, 0, (long)str);
		sprintf(str, "%d", WxGetLeftShift());
		SendDlgItemMessage(hDlg, IDC_LEFTSHIFT, WM_SETTEXT, 0, (long)str);
		sprintf(str, "%d", WxGetSampleRate());
		SendDlgItemMessage(hDlg, IDC_SAMPLERATE, WM_SETTEXT, 0, (long)str);
		sprintf(str, "%d", WxGetCutoffFreq());
		SendDlgItemMessage(hDlg, IDC_CUTOFFFREQ, WM_SETTEXT, 0, (long)str);
		SendDlgItemMessage(hDlg,
					WxGetPassDir() == PD_NORTH ? IDC_RADIONORTH : IDC_RADIOSOUTH,
					BM_SETCHECK, 1, 0L);
		SendDlgItemMessage(hDlg, IDC_CHECKSKIPLINES, BM_SETCHECK,
					WxGetLineDecimate() > 1 ? 1 : 0, 0L);
		SendDlgItemMessage(hDlg, IDC_CHECKPROCRT, BM_SETCHECK,
					WxGetProcRT(), 0L);
		SendDlgItemMessage(hDlg, IDC_CHECKSAVEWAV, BM_SETCHECK,
					WxGetSaveWav(), 0L);

		return TRUE;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case CM_SAVE :
		case IDOK :
			GetWindowText(GetDlgItem(hDlg, IDC_LINEPERIOD), str, 16);
			WxSetLinePeriod(atof(str));
			GetWindowText(GetDlgItem(hDlg, IDC_CONTRAST), str, 16);
			WxSetContrast(atof(str));
			GetWindowText(GetDlgItem(hDlg, IDC_BRIGHTNESS), str, 16);
			WxSetBrightness(atoi(str));
			GetWindowText(GetDlgItem(hDlg, IDC_LEFTSHIFT), str, 16);
			WxSetLeftShift(atoi(str));
			GetWindowText(GetDlgItem(hDlg, IDC_SAMPLERATE), str, 16);
			WxSetSampleRate(atoi(str));
			GetWindowText(GetDlgItem(hDlg, IDC_CUTOFFFREQ), str, 16);
			WxSetCutoffFreq(atoi(str));
			if (SendDlgItemMessage(hDlg, IDC_RADIONORTH, BM_GETCHECK, 0, 0L))
				WxSetPassDir(PD_NORTH);
			else
				WxSetPassDir(PD_SOUTH);

			if (SendDlgItemMessage(hDlg, IDC_CHECKSKIPLINES, BM_GETCHECK, 0, 0L))
				WxSetLineDecimate(SCRNLINEDECIMATE);
			else
				WxSetLineDecimate(1);

			WxSetProcRT(SendDlgItemMessage(hDlg, IDC_CHECKPROCRT,
											BM_GETCHECK, 0, 0L));

			WxSetSaveWav(SendDlgItemMessage(hDlg, IDC_CHECKSAVEWAV,
											BM_GETCHECK, 0, 0L));


			if (LOWORD(wParam) == CM_SAVE)
				WxSaveOptions();
			else
				EndDialog(hDlg, IDOK);
			break;

		case IDCANCEL:
			EndDialog(hDlg, IDOK);
			return TRUE;
		}
		break;
	}
	return FALSE;
}


////////////////////////////////////////////////////////////
// Handle the WM_PAINT message
////////////////////////////////////////////////////////////
#pragma argsused
void Wx_OnPaint(HWND hwnd)
{
	PAINTSTRUCT PaintStruct;
	int NumImages = 15;

	HDC PaintDC = BeginPaint(hwnd, &PaintStruct);
	HDC BitmapDC = CreateCompatibleDC(PaintDC);
	HBITMAP ScrnBitmap = GetWxScrnBitmap();
	HBITMAP OldBitmap;

	if (ScrnBitmap == 0) {
		OldBitmap = SelectBitmap(BitmapDC, TheBitmap);
		for (int i = 0; i < NumImages; i++)
			for (int j = 0; j < NumImages; j++)
				BitBlt(PaintDC, i * 64, j * 64, 64, 64, BitmapDC, 0, 0, SRCCOPY);
		SelectBitmap(BitmapDC, OldBitmap);
	}

	if (ScrnBitmap != 0) {
		WxPaint();
	}

	DeleteDC(BitmapDC);
	EndPaint(hwnd, &PaintStruct);
}

void SaveAs(HWND hwnd)
{
	char szFile[256];
	char *def = "noaa.bmp";
	strcpy(szFile, def);
	if (GetSaveName(hwnd, szFile, sizeof(szFile), "Bitmap Files(*.bmp)", "*.bmp")) {
		rename(def, szFile);
	}
}


void PostProcess(HWND hwnd)
{
	void DoWavFile(char *szFileName, HWND hwndApp);
	char szFile[256];
	strcpy(szFile, "noaa.wav");
	if (GetOpenName(hwnd, szFile, sizeof(szFile), "Wave Files(*.wav)", "*.wav")) {
		// Do Hourglass cursor, this could be awhile
		HCURSOR Old = SetCursor(LoadCursor(0, IDC_WAIT));
		DoWavFile(szFile, hwnd);
		SetCursor(Old);
	}
}

void ReadBitmap(HWND hwnd)
{
	BMPinfo *lpBmpInfo;
	char szFile[256];
	strcpy(szFile, "noaa.bmp");
	GetOpenName(hwnd, szFile, sizeof(szFile), "Bitmap Files(*.bmp)", "*.bmp");
	lpBmpInfo = OpenReadBmpFile(szFile);
	CloseReadBmpFile(lpBmpInfo);
}


void RealTime(HWND hwnd)
{
	static PCMWAVEFORMAT pcmWaveFormat;

	if (!waveInputActive()) {
		pcmWaveFormat.wBitsPerSample = BITSPERSAMPLE;
		pcmWaveFormat.wf.wFormatTag = WAVE_FORMAT_PCM;
		pcmWaveFormat.wf.nChannels = 1;
		pcmWaveFormat.wf.nSamplesPerSec = WxGetSampleRate();
		pcmWaveFormat.wf.nAvgBytesPerSec = WxGetSampleRate();
		pcmWaveFormat.wf.nBlockAlign = 1;

		lpWaveFile->hwndApp = hwnd;
		lpWaveFile->pFormat = (WAVEFORMAT *)&pcmWaveFormat;

		if (WxGetSaveWav()) 
      openWriteWaveFile("noaa.wav", lpWaveFile);

		/* init the bitmap display routines */
		if (WxGetProcRT()) {
			InitBitmap(hwnd);
			InitWxFileBitmap(100);  // we don't know the real size,
														 //    but CloseBmpFile will fix it
		}

		recordWaveDevice(hwnd, WxGetSampleRate(), BITSPERSAMPLE,
														CHANNELS, WxGetSampleRate()*2);
	}
}

#pragma argsused
void StopRealTime(HWND hwnd)
{
	if (waveInputActive()) {
		if (WxGetSaveWav()) 
		  closeWriteWaveFile(lpWaveFile);
		closeInWaveDevice();
		closeWxFileBitmap();
	}
}

void SaveWaveBlock(LPBYTE lpData, DWORD dwSize)
{
	lpWaveFile->lpData = lpData;
	lpWaveFile->dwDataSize = dwSize;
	writeWaveFileData(lpWaveFile);
}

#pragma argsused
void Wx_OnWim_Data(HWND hwnd, HWAVEIN hDev, LPWAVEHDR lpHdr)
{
	void ScanConv(LPBYTE lpBuf, int nBufSize);

	if(waveInputActive()) {
 		if (WxGetSaveWav()) 
		  SaveWaveBlock((LPBYTE)lpHdr->lpData, lpHdr->dwBytesRecorded);
		if (WxGetProcRT())
			ScanConv((LPBYTE)lpHdr->lpData, (int)lpHdr->dwBytesRecorded);
		continueWaveInput(hDev, lpHdr);
	}
}

#pragma argsused
void Wx_OnWim_Close(HWND hwnd)
{
	// be sure real time has been stoped
	StopRealTime(hwnd);
}

