#define STRICT

#include <windows.h>
#include <windowsx.h>
#include <string.h>
#include <stdio.h>
#pragma hdrstop
#include <math.h>
#include "bmp.h"
#include "wxbmp.h"
#include "defines.h"

/* Macro to round UP the given value in bits to the next larger byte */
#define WIDTHBYTES(i)	((i+31)/32*4)
#define PALVERSION      0x300

static int nBmpWidth;
static int nBmpByteWidth;
static int nBmpFileWidth;
static int nBmpFileByteWidth;
static float fLineTime;

static const int nBmpHeight = 1;
static const int nScrnBmpHeight = SCRNBMPHEIGHT;

// scroll bar
static int nHPos = 0;
static int nHMax;
static int nHStep;
static int nHPage;

static int nVPos = 0;
static int nVMax;
static int nVStep;
static int nVPage;

static HANDLE hBitArray = 0, hFileBitArray = 0;
static BYTE *lpBits, *lpBitArray;
static BYTE *lpBmpFileBits, *lpFileBitArray;
static short xStart, yStart, yGoal;
static int nWinTop, nWinBottom;
static int nPassDir;
static BITMAP bm;
static HDC hdc = 0, hdcMem = 0, hdcScrnMem = 0;
static HBITMAP hBitmap = 0, hScrnBitmap = 0;
static HBITMAP hPrevBitmap = 0, hPrevScrnBitmap = 0;
static POINT ptSize, ptOrg;
static float fSGoal, fSamplesPerLine;
static DWORD dwSampleCount, dwSGoal;
static HANDLE hBi = 0;
static HPALETTE hPal = 0;
static HPALETTE hPrevPal = 0;

const int nLutSize = 256;
const int nGrays = 32;   // 32 thats all the grays available on most cards
static BYTE bLut[nLutSize];
static DWORD dwHist[nLutSize];
static RGBQUAD rgbMap[nLutSize];

static BMPinfo bmpInfo;

// nHPos is the left edge (from the horz. scroller)
void WxPaint(void)
{
	if (hScrnBitmap) {
		BITMAP bm;
		GetObject(hScrnBitmap, sizeof(BITMAP), (LPSTR)&bm);
		BitBlt(hdc, 0, 0, bm.bmWidth, bm.bmHeight,
					 hdcScrnMem, nHPos, nVPos, SRCCOPY);
	}
}

inline
int max(int x, int y)
{
	return (x > y ? x : y);
}
inline
int min(int x, int y)
{
	return (x < y ? x : y);
}

void Wx_OnHScroll(HWND hwnd, HWND hwndCtl, UINT code, int pos)
{
	switch(code) {
		case SB_LEFT :      nHPos = 0; break;
		case SB_RIGHT :     nHPos = nHMax; break;
		case SB_LINELEFT :  nHPos -= nHStep; break;
		case SB_LINERIGHT : nHPos += nHStep; break;
		case SB_PAGELEFT :  nHPos -= nHPage; break;
		case SB_PAGERIGHT : nHPos += nHPage; break;
		case SB_THUMBPOSITION: nHPos = pos; break;
	}

	nHPos = max(0, min(nHPos, nHMax));
	SetScrollPos(hwnd, SB_HORZ, nHPos, TRUE);
	InvalidateRect(hwnd, NULL, FALSE);
}

void Wx_OnVScroll(HWND hwnd, HWND hwndCtl, UINT code, int pos)
{
	switch(code) {
		case SB_TOP :      nVPos = 0; break;
		case SB_BOTTOM :   nVPos =  nVMax; break;
		case SB_LINEUP :   nVPos -= nVStep; break;
		case SB_LINEDOWN : nVPos += nVStep; break;
		case SB_PAGEUP :   nVPos -= nVPage; break;
		case SB_PAGEDOWN : nVPos += nVPage; break;
		case SB_THUMBPOSITION: nVPos = pos; break;
	}

	nVPos = max(0, min(nVPos, nVMax));
	SetScrollPos(hwnd, SB_VERT, nVPos, TRUE);
	InvalidateRect(hwnd, NULL, FALSE);
}


HBITMAP GetWxScrnBitmap(void)
{
	return(hScrnBitmap);
}

void InitBitmap(HWND hwnd)
{
	static int nLastWidth = 0;
	void InitLut(void);
	short nHeight = 1;
	RECT rect;

	nBmpWidth = (WxGetSampleRate() * WxGetLinePeriod()) / WxGetScrnDecimate();
	nBmpByteWidth = WIDTHBYTES(nBmpWidth * 8);
	nBmpFileWidth = (WxGetSampleRate() * WxGetLinePeriod()) / WxGetFileDecimate();
	nBmpFileByteWidth = WIDTHBYTES(nBmpFileWidth * 8);

	GetClientRect(hwnd, &rect);

	nHPage = rect.right - rect.left;
	nHStep = nHPage/8;
	nHMax = nBmpWidth - nHPage;

	nVPage = rect.bottom - rect.top;
	nVStep = nVPage/8;
	nVMax = nScrnBmpHeight - nVPage;

	SetScrollRange(hwnd, SB_HORZ, 0, nHMax, FALSE);
	SetScrollRange(hwnd, SB_VERT, 0, nVMax, FALSE);


	if (nLastWidth != nBmpWidth) {
		nLastWidth = nBmpWidth;

		if (hBitArray) {
			LocalUnlock(hBitArray);
			LocalFree(hBitArray);
		}
		if ((hBitArray = LocalAlloc(LHND, nBmpByteWidth)) != 0)
			lpBitArray = (BYTE *)LocalLock(hBitArray);

		if (hFileBitArray) {
			LocalUnlock(hFileBitArray);
			LocalFree(hFileBitArray);
		}
		if ((hFileBitArray = LocalAlloc(LHND, nBmpFileByteWidth)) != 0)
			lpFileBitArray = (BYTE *)LocalLock(hFileBitArray);

		if (!lpBitArray || !lpFileBitArray) {
			MessageBox(hwnd, "Not Enough Memory to Alloc Bit Arrays", NULL,
									MB_ICONSTOP | MB_OK);
			return;
		}

		if (!hdc)
			hdc = GetDC(hwnd);

		SetMapMode(hdc, MM_TEXT);

		if (hBi)
			GlobalFree(hBi);
		if ((hBi = CreateDibBitmapInfo(nBmpWidth, nHeight, nGrays)) == NULL) {
			MessageBox(hwnd, "Can't CreateDibBitmap", "Wx", MB_ICONSTOP | MB_OK);
			return;
		}

		if (hPal) {
			SelectPalette(hdc, hPrevPal, FALSE);
			DeleteObject(hPal);
		}
		if ((hPal = CreateDibPalette(hBi)) == NULL) {
			MessageBox(hwnd, "Can't CreateDibPalette", "Wx", MB_ICONSTOP | MB_OK);
			return;
		}

		hPrevPal = SelectPalette(hdc, hPal, FALSE);
		RealizePalette(hdc);

		// create small "memory" bitmap to blt to screen and Scrn mem bitmap
		if (!hdcMem)
			hdcMem = CreateCompatibleDC(hdc);

		if (hBitmap) {
			SelectObject(hdcMem, hPrevBitmap);
			DeleteBitmap(hBitmap);
		}
		hBitmap = CreateCompatibleBitmap(hdc, nBmpWidth, nHeight);
		hPrevBitmap = (HBITMAP)SelectObject(hdcMem, hBitmap);
		SetMapMode(hdcMem, GetMapMode(hdc));

		// create Scrn mem bitmap to save screen for repaint ect.
		if (!hdcScrnMem)
			hdcScrnMem = CreateCompatibleDC(hdc);

		if (hScrnBitmap) {
			SelectObject(hdcScrnMem, hPrevScrnBitmap);
			DeleteBitmap(hScrnBitmap);
		}
		hScrnBitmap = CreateCompatibleBitmap(hdc, nBmpWidth, nScrnBmpHeight);
		hPrevScrnBitmap = (HBITMAP)SelectObject(hdcScrnMem, hScrnBitmap);
		SetMapMode(hdcScrnMem, GetMapMode(hdc));


		// set variables to size of the bitmap used in bitblt
		GetObject(hBitmap, sizeof(BITMAP), (LPSTR)&bm);
		ptSize.x = bm.bmWidth;
		ptSize.y = bm.bmHeight;
		ptOrg.x = ptOrg.y = 0;

	}

	// clear the off scrn bitmap
	PatBlt(hdcScrnMem, 0, 0, nBmpWidth, nScrnBmpHeight, WHITENESS);
	// clear the scrn 
	WxPaint();

	xStart = 0;
	nWinTop = rect.top;
	nWinBottom = rect.bottom;
	if (WxGetPassDir() == PD_SOUTH) {
		nPassDir = -1;
		yStart = nScrnBmpHeight;
		yGoal =  0;
		lpBits = &lpBitArray[nBmpWidth-1];
		nVPos = nVMax;
	}
	else {
		nPassDir = 1;
		yStart =  0;
		yGoal = nScrnBmpHeight;
		lpBits = lpBitArray;
		nVPos = 0;
	}
	SetScrollPos(hwnd, SB_VERT, nVPos, TRUE);

	fLineTime = WxGetLinePeriod();
	fSGoal = fSamplesPerLine = (fLineTime * WxGetSampleRate());
	dwSGoal = (int)fSamplesPerLine;
  dwSampleCount = 0;

	InitLut();
}

void InitLut(void)
{
	// conv 0-255 to 0.0-1.0
	// sqrt
	// conv  to 0-31
	BYTE ByteMin(int, BYTE);

	for (int i = 0; i < nLutSize; i++) {
		float p = (float)i / (nLutSize-1);
		bLut[i] = ByteMin((int)(p*WxGetContrast()*(float)(nGrays-1)), nGrays-1);
		rgbMap[i].rgbRed = rgbMap[i].rgbGreen = rgbMap[i].rgbBlue = i;
		dwHist[i] = 0;
	}
}


void InitWxFileBitmap(int n)
{
	if (bmpInfo.file != NULL)
		CloseWriteBmpFile(&bmpInfo);

	if (WxGetPassDir() == PD_SOUTH) {
		lpBmpFileBits = &lpFileBitArray[nBmpFileWidth-1];
	}
	else {
		lpBmpFileBits = lpFileBitArray;
	}

	bmpInfo.w = nBmpFileWidth;
	bmpInfo.h = n;
	bmpInfo.bw = nBmpFileByteWidth;
	bmpInfo.cmap_n = 256;
	bmpInfo.cmap = rgbMap;
	OpenWriteBmpFile("noaa.bmp", &bmpInfo);
}

// scale 0-255 to 0-32
BYTE ScalePixel(BYTE p)
{
	dwHist[p]++;
	return(p >> 3);
//	return(bLut[p]);
}

int lC = 0;

void AddBitmapPixel(BYTE bPixel)
{
  static int nCount = 0;

	BYTE c = ScalePixel(bPixel);
	*lpBmpFileBits = bPixel;
	lpBmpFileBits += nPassDir;
	if (++nCount == DECIMATERATIO) {
		nCount = 0;
		*lpBits = c;
		lpBits += nPassDir;
		lC++;
	}
}

void NewLine(void)
{
	// do memory and screen bitmaps
	DWORD dwCount = sizeof(lpBitArray);
	static int nLineCount = 0;

	lC = 0;
	if (yStart != yGoal) {
		/* new line done */
		if (++nLineCount == WxGetLineDecimate()) {
			nLineCount = 0;

			// FOR DEBUG: draw color map at the begining of the lines
		 //	for (int i = 0; i < 256; i++)
		 //	 lpBitArray[i] = i/8;

			/* set bits in the small bitmap */
			LPBITMAPINFO lpbi = (LPBITMAPINFO)GlobalLock(hBi);
			if (!lpbi)
				return;
			SetDIBits(hdc, hBitmap, 0, 1, lpBitArray, lpbi, DIB_PAL_COLORS);
			GlobalUnlock(hBi);

			//search bits for start of visible scan
			/*
			const int window = 16;
			int xpos = 0;
			for (int i = 0; xpos == 0 && i < ptSize.x-window; i++) {
				long sum = 0;
				if (lpBitArray[i] > 25 && lpBitArray[i+1] < 15) {
					for (int j = 0; j < window; j++)
						 sum += lpBitArray[i+j];
					if (sum < window*15)
						xpos = i;
				}
			}
			*/
			/* blt onto screen and into Scrn bitmap */
			int y = yStart - nVPos;
			if (y > nWinTop && y < nWinBottom)
				BitBlt(hdc, xStart, y, ptSize.x, ptSize.y,
									hdcMem, nHPos, ptOrg.y, SRCCOPY);

			BitBlt(hdcScrnMem, xStart, yStart, ptSize.x, ptSize.y,
								hdcMem, ptOrg.x, ptOrg.y, SRCCOPY);
			yStart += nPassDir;
		}
	}
	// do file bitmap
	WriteBmp(&bmpInfo, (char *)lpFileBitArray, 1);

	if (nPassDir == -1) {
		lpBits = &lpBitArray[nBmpWidth-1];
		lpBmpFileBits = &lpFileBitArray[nBmpFileByteWidth-1];
	}
	else {
		lpBits = lpBitArray;
		lpBmpFileBits = lpFileBitArray;
	}


}

void closeWxFileBitmap(void)
{
	CloseWriteBmpFile(&bmpInfo);
}

int NewSample(void)
{
	if (dwSampleCount++ >= dwSGoal) {
		NewLine();
		fSGoal += fSamplesPerLine;
		dwSGoal = (DWORD)fSGoal;
		return 1;
	}
	return 0;
}

void DestroyBitmap(void)
{
	SelectObject(hdcMem, hPrevBitmap);
	DeleteDC(hdcMem);
	SelectObject(hdcScrnMem, hPrevScrnBitmap);
	DeleteDC(hdcScrnMem);
	SelectPalette(hdc, hPrevPal, FALSE);
	DeleteDC(hdc);
	DeleteBitmap(hBitmap);
	DeleteBitmap(hScrnBitmap);
	GlobalFree(hBi);
	DeleteObject(hPal);
	LocalUnlock(hBitArray);
	LocalFree(hBitArray);
	LocalUnlock(hFileBitArray);
	LocalFree(hFileBitArray);
}

/****************************************************************************
 *
 *  FUNCTION   : CreateDibBitmapInfo
 *
 *  PURPOSE    : Creates a DIB with palette of index's NOT RGB's
 *      we must use DIB_PAL_COLORS as the wUsage parameter to SetDIBits()
 *
 *  RETURNS    : A handle to the BITMAPINFO of the DIB
 *
 ****************************************************************************/
HANDLE CreateDibBitmapInfo(DWORD dwWidth, DWORD dwHeight, WORD nNumColors)
{
	 DWORD off;
	 HANDLE hbi = NULL;
	 int size;
	 int i;
	 WORD FAR *pCT;
	 BITMAPINFOHEADER bi;
	 BITMAPCOREHEADER bc;
	 LPBITMAPINFOHEADER lpbi;
	 WORD wPlanes, wBitCount;

	 wPlanes = 1;
	 wBitCount = 8;

	 bi.biSize = sizeof(BITMAPINFOHEADER);
	 bi.biWidth = dwWidth;
	 bi.biHeight = dwHeight;
	 bi.biPlanes = wPlanes;
	 bi.biBitCount = wBitCount;
	 bi.biCompression = BI_RGB;
	 bi.biSizeImage = WIDTHBYTES ((DWORD)bi.biWidth * bi.biBitCount) * bi.biHeight;
	 bi.biXPelsPerMeter = 0;
	 bi.biYPelsPerMeter = 0;
	 bi.biClrUsed = nNumColors;
	 bi.biClrImportant = nNumColors;

	 /* Allocate for the BITMAPINFO structure and the color table. */
	 hbi = GlobalAlloc(GHND, (LONG)bi.biSize + nNumColors * sizeof(WORD));
	 if (!hbi)
			return NULL;
	 lpbi = (LPBITMAPINFOHEADER)GlobalLock(hbi);
	 *lpbi = bi;

	 /* Get a pointer to the color table */
	 pCT = (WORD FAR *)((LPSTR)lpbi + bi.biSize);
	 /* Fill it with palette indes */
	 for (i = 0; i < nNumColors; i++)
			pCT[i] = i;

	 GlobalUnlock(hbi);
	 return hbi;
}

/****************************************************************************
 *                                                                          *
 *  FUNCTION   : CreateBIPalette(LPBITMAPINFOHEADER lpbi)                   *
 *                                                                          *
 *  PURPOSE    : Given a Pointer to a BITMAPINFO struct will create a       *
 *               a GDI palette object of Grays.                             *
 *                                                                          *
 *  RETURNS    : A handle to the palette.                                   *
 *                                                                          *
 ****************************************************************************/
HPALETTE CreateBIPalette(LPBITMAPINFOHEADER lpbi)
{
   LOGPALETTE *pPal;
   HPALETTE hpal = NULL;
   WORD nNumColors;
   BYTE red;
   BYTE green;
   BYTE blue;
   WORD i;
	 RGBQUAD FAR *pRgb;
	 int c, cStep;

   if (!lpbi)
      return NULL;
   if (lpbi->biSize != sizeof(BITMAPINFOHEADER))
      return NULL;

   /* Get a pointer to the color table and the number of colors in it */
   pRgb = (RGBQUAD FAR *)((LPSTR)lpbi + (WORD)lpbi->biSize);
	 nNumColors = (WORD)lpbi->biClrUsed;
   if (nNumColors)
   {
      /* Allocate for the logical palette structure */
      pPal = (LOGPALETTE *)LocalAlloc(LPTR, sizeof(LOGPALETTE) + nNumColors *
																			sizeof(PALETTEENTRY));
			if (!pPal) {
				MessageBox(NULL, "Can't Alloc Palette Memory", "Wx", MB_ICONSTOP | MB_OK);
				return NULL;
			}

			pPal->palNumEntries = nNumColors;
			pPal->palVersion = PALVERSION;

      /* Fill in the palette entries from the DIB color table and
       * create a logical color palette.
			 */
	/*		if (nNumColors == 32) {
				c = 69;
				cStep = 6;
			}
			else */{
				c = 0;
				cStep = 256 / nNumColors;
			}
			for (i = 0; i < nNumColors; i++, c += cStep)
			{
				if (c > 255) c = 255;
				 pPal->palPalEntry[i].peRed = c;
				 pPal->palPalEntry[i].peGreen = c;
				 pPal->palPalEntry[i].peBlue = c;
         pPal->palPalEntry[i].peFlags = (BYTE)0;
      }
      hpal = CreatePalette(pPal);
      LocalFree((HANDLE)pPal);
   }
	 return hpal;
}

/****************************************************************************
 *                                                                          *
 *  FUNCTION   : CreateDibPalette(HANDLE hbi)                               *
 *                                                                          *
 *  PURPOSE    : Given a Global HANDLE to a BITMAPINFO Struct               *
 *               will create a GDI palette object from the color table.     *
 *               (BITMAPINFOHEADER format DIBs only)                                     *
 *                                                                          *
 *  RETURNS    : A handle to the palette.                                   *
 *                                                                          *
 ****************************************************************************/


HPALETTE CreateDibPalette (HANDLE hbi)
{
   HPALETTE hpal;

   if (!hbi)
			return NULL;
   hpal = CreateBIPalette((LPBITMAPINFOHEADER)GlobalLock(hbi));
   GlobalUnlock(hbi);
   return hpal;
}



